@@ -5,7 +5,6 @@
 
 use ffi_sdk::Platform;
 
-// use rand::Rng;
 use super::*;
 use crate::identity::Identity;
 
@@ -86,11 +85,11 @@ impl DittoBuilder {
     pub fn with_identity<F, I>(mut self, factory: F) -> Result<Self, DittoError>
     where
         F: FnOnce(Arc<dyn DittoRoot>) -> Result<I, DittoError>,
-        I: Identity + 'static, // must returned something ownable
+        I: Identity + 'static, // must return something ownable
     {
         match &self.ditto_root {
             Some(root) => {
-                let identity = factory(root.clone())?;
+                let identity = factory(root.retain())?;
                 self.identity = Some(Arc::new(identity));
             }
             None => {
@@ -110,7 +109,7 @@ impl DittoBuilder {
     {
         match &self.identity {
             Some(id) => {
-                let config = factory(id.clone());
+                let config = factory(id.retain());
                 self.transport_config = Some(config)
             }
             None => {
@@ -139,7 +138,7 @@ impl DittoBuilder {
         let boxed_ditto = unsafe { ffi_sdk::ditto_make(uninit_ditto, auth_client_ref) };
         let ditto: Arc<BoxedDitto> = Arc::new(boxed_ditto);
         let site_id: SiteId = identity.site_id();
-        let store = Store::new(ditto.clone(), site_id);
+        let store = Store::new(ditto.retain());
         let mut transport_config = self.transport_config.unwrap_or_else(|| {
             let mut config = TransportConfig::new();
             config.enable_all_peer_to_peer();
@@ -155,24 +154,50 @@ impl DittoBuilder {
         }
         let transports: Arc<RwLock<Transports>> = Arc::new(RwLock::new(Transports::from_config(
             transport_config,
-            ditto.clone(),
+            ditto.retain(),
         )));
 
         let auth = identity.authenticator();
-        let validity_listener = identity.make_listener(transports.clone());
+        let validity_listener = identity.make_listener(transports.retain());
         let presence_manager_v2 = Arc::new(PresenceManagerV2Context::new());
 
-        let sdk_ditto = Ditto {
+        let fields_from_auth = |auth| DittoFields {
             ditto,
             auth,
             store,
-            activated: false,
+            activated: false.into(),
             site_id,
             transports,
             ditto_root,
             validity_listener,
             presence_manager_v2,
         };
+        let fields = if let Some(mut auth) = auth {
+            // Use polyfilled method until `arc_new_cyclic` gets stabilized.
+            use crate::utils::extension_traits::polyfills::ArcNewCyclic;
+            #[allow(unstable_name_collisions)]
+            Arc::new_cyclic(|weak_fields: &arc::Weak<_>| {
+                #[rustfmt::skip]
+                // Remove this once we no longer use the polyfill.
+                let weak_fields = unsafe {
+                    // Safety:
+                    //   - same layout,
+                    //   - we never upgrade the weak within this scope.
+                    crate::utils::transmute_by_ref::<
+                        arc::Weak<::core::mem::MaybeUninit<DittoFields>>,
+                        arc::Weak<DittoFields>,
+                    >(weak_fields)
+                };
+                auth.ditto_fields = weak_fields.clone();
+                fields_from_auth(Some(auth))
+            })
+        } else {
+            Arc::new(fields_from_auth(None))
+        };
+
+        let sdk_ditto = Ditto {
+            fields: ManuallyDrop::new(fields),
+        };
         Ok(sdk_ditto)
     }
 }
